"""Automatically generated by hassfest.

To update, run python3 -m hassfest
"""


SSDP = {
    "device_type": {},
    "manufacturer": {
        "Royal Philips Electronics": [
            "hue"
        ]
    },
    "st": {}
}
