"""Automatically generated by hassfest.

To update, run python3 -m hassfest
"""


ZEROCONF = {
    "_axis-video._tcp.local.": [
        "axis"
    ],
    "_coap._udp.local.": [
        "tradfri"
    ],
    "_esphomelib._tcp.local.": [
        "esphome"
    ],
    "_hap._tcp.local.": [
        "homekit_controller"
    ]
}

HOMEKIT = {
    "LIFX ": "lifx"
}
